@@ -1,7 +1,7 @@
-import json
-import uuid
+from typing import List, Iterator, Union, Optional
+import os
 from .api_dto import ApiDto
-from enum import Enum
+from datetime import datetime, timezone
 
 
 def get_bool(obj, name: str):
@@ -11,11 +11,335 @@ def get_bool(obj, name: str):
         return bool(obj[name])
 
 
+class ModelIdentifierInfo:
+    """
+    define metadata associated to an identifier (regardless of its alias)
+    """
+
+    def __init__(self,
+                 identifier: str = None,
+                 active_alias: str = None):
+        self.identifier = identifier
+        self.active_alias = active_alias
+
+    def to_json(self):
+        obj = {}
+        if self.active_alias is not None:
+            obj["active_alias"] = self.active_alias
+        return obj
+
+    def from_json(self, json_dict):
+        if "active_alias" in json_dict:
+            self.active_alias = json_dict["active_alias"]
+
+class ModelFile:
+    """
+    define a model file.
+
+    :ivar path str: file name relative path e.g. trained_model.pkl
+    :ivar content bytes[]: array of content bytes
+    """
+
+    def __init__(self,
+                 full_path: str = None,
+                 path: str = None,
+                 name: str = None,
+                 extension: str = None,
+                 last_modified: datetime = None,
+                 size: int = None,
+                 content = None):
+        self.full_path = full_path
+        self.name = name
+        self.extension = extension
+        self.path = path
+        self.last_modified = last_modified
+        self.size = size
+        self.content = content
+
+
+    @property
+    def path(self) -> Optional[str]:
+        if self.name and self.extension:
+            return f"{self.name}.{self.extension}"
+        return self.name or None
+
+    @path.setter
+    def path(self, value: Optional[str]):
+        if not value:
+            self.name = None
+            self.extension = None
+            return
+
+        base = os.path.basename(value.strip())
+        name_part, ext = os.path.splitext(base)
+        self.name = name_part or None
+        self.extension = ext.lstrip(".").lower() if ext else None
+
+    def to_json(self):
+        """
+        convert this entity in a dict that can be json serializable
+        :return: dict
+        """
+        obj = {}
+        if self.full_path is not None:
+            obj["full_path"] = str(self.full_path)
+        if self.name is not None:
+            obj["name"] = self.name
+        if self.extension is not None:
+            obj["extension"] = self.extension
+        if self.path is not None:
+            obj["path"] = self.path
+        if self.last_modified is not None:
+            obj["last_modified"] = self.last_modified.timestamp() * 1000.0
+        if self.size is not None:
+            obj["size"] = self.size
+        return obj
+
+    def from_json(self, obj):
+        """
+        load this entity from a dict
+        """
+        if "full_path" in obj.keys():
+            self.full_path = obj["full_path"]
+        if "name" in obj.keys():
+            self.name = obj["name"]
+        if "extension" in obj.keys():
+            self.extension = obj["extension"]
+        if not self.name and not self.extension and "path" in obj:
+            self.path = obj["path"]
+        if "last_modified" in obj.keys():
+            self.last_modified = datetime.fromtimestamp(obj["last_modified"] / 1000, tz=timezone.utc)
+        if "size" in obj.keys():
+            self.size = obj["size"]
+
+
+class ModelInfo:
+    """
+    define a pointer to a machine learning model.
+
+    :ivar model_type str: mlflow model flavour
+    :ivar files list: list of ModelFile of model content.
+    """
+
+    def __init__(self,
+                 key: str,
+                 twin_hardware_id: str = None,
+                 property_value: str = None,
+                 alias: str = None,
+                 model_type: str = None,
+                 file_format: str = 'pkl',
+                 source: str = 'wizata',
+                 property_name: str = None,
+                 trained_model = None,
+                 scaler = None,
+                 files = None
+                 ):
+        # information identifying a model
+        self.key = key
+        self.twin_hardware_id = twin_hardware_id
+        self.property_value = property_value
+        self.alias = alias
+        self.file_format = file_format
+        if self.file_format not in ["pt", "pkl"]:
+            raise ValueError("file_format must be 'pt' or 'pkl'")
+        self.source = source
+        self.model_type = model_type
+        if files is None:
+            self.files = []
+        else:
+            self.files = files
+            for file in files:
+                if not isinstance(file, ModelFile):
+                    raise TypeError("file is not a ModelFile with files")
+        self.is_active = False
+
+        # files attached to model when loaded
+        self.trained_model = trained_model
+        self.scaler = scaler
+
+        # temporary properties during model generation not generally stored
+        self.property_name = property_name
+        self.input_columns = None
+        self.has_target_feat = False
+        self.label_counts = 0
+
+    @classmethod
+    def split_identifier(cls, identifier: str):
+        """
+        split the identifier into four parts
+        :param identifier: identifier.
+        :return: key, twin, property, alias
+        """
+
+        # extract alias
+        if "@" in identifier:
+            split_identifier_for_alias = identifier.split("@")
+            identifier = split_identifier_for_alias[0]
+            alias = split_identifier_for_alias[1]
+        else:
+            alias = None
+
+        # extract key, twin, property
+        identifiers = identifier.split('.')
+        key = identifiers[0]
+        twin_hardware_id = None
+        if len(identifiers) > 1:
+            twin_hardware_id = identifiers[1]
+        property_value = None
+        if len(identifiers) > 2:
+            property_value = identifiers[2]
+
+        return key, twin_hardware_id, property_value, alias
+
+    def identifier(self, include_alias: bool = False) -> str:
+        """
+        returns the complete string identifier for this model.
+        :param include_alias: include the alias pointer or leave it to target the default version.
+        :return: complete identifier of a model.
+        """
+        if self.key is None:
+            raise KeyError('please specific a model key')
+        identifier = self.key
+
+        if self.twin_hardware_id is not None:
+            identifier += f".{self.twin_hardware_id}"
+
+        if self.property_value is not None:
+            identifier += f".{self.property_value}"
+
+        if include_alias and self.alias is not None:
+            identifier += f"@{self.alias}"
+
+        return identifier
+
+    def add_file(self, file: ModelFile):
+        """
+        add a path to list of known path.
+        detect depending on file type further actions.
+        :param file: ModelFile
+        :return: None
+        """
+        if file.name == "trained_model":
+            self.file_format = file.extension
+        self.files.append(file)
+
+    def to_json(self) -> dict:
+        """
+        convert this entity in a dict that can be json serializable
+        :return: dict
+        """
+        obj = {
+            "key": self.key,
+            "file_format": self.file_format,
+            "source": self.source
+        }
+        if self.twin_hardware_id is not None:
+            obj["twin_hardware_id"] = str(self.twin_hardware_id)
+        if self.property_value is not None:
+            obj["property_value"] = self.property_value
+        if self.alias is not None:
+            obj["alias"] = self.alias
+        if self.model_type is not None:
+            obj["model_type"] = self.model_type
+        if self.property_name is not None:
+            obj["property_name"] = self.property_name
+        if self.is_active is not None:
+            obj["is_active"] = self.is_active
+        if self.identifier is not None:
+            obj["identifier"] = self.identifier(include_alias=True)
+        if len(self.files) > 0:
+            obj["files"] = [file.to_json() for file in self.files]
+        return obj
+
+    def from_json(self, obj):
+        """
+        load this entity from a dict
+        """
+        if "key" in obj.keys():
+            self.key = obj["key"]
+        if "twin_hardware_id" in obj.keys():
+            self.twin_hardware_id = obj["twin_hardware_id"]
+        if "property_value" in obj.keys():
+            self.property_name = obj["property_value"]
+        if "alias" in obj.keys():
+            self.alias = obj["alias"]
+        if "model_type" in obj.keys():
+            self.model_type = obj["model_type"]
+        if "file_format" in obj.keys():
+            self.file_format = obj["file_format"]
+        if "source" in obj.keys():
+            self.source = obj["source"]
+        if "property_name" in obj.keys():
+            self.property_name = obj["property_name"]
+        if "is_active" in obj.keys():
+            self.is_active = get_bool(obj, name="is_active")
+
+
+class ModelList:
+    """
+    used to conveniently manipulate a list of models.
+    """
+
+    def __init__(self):
+        self.models: List[ModelInfo] = []
+        self.identifiers_info = {}
+
+    def __iter__(self) -> Iterator[ModelInfo]:
+        return iter(self.models)
+
+    def __len__(self) -> int:
+        return len(self.models)
+
+    def exists(self, model: ModelInfo) -> bool:
+        """
+        check if a model is within the model list.
+        :param model: identifier to check.
+        :return: True if it exists.
+        """
+        return any(model_in_list.identifier(include_alias=True) == model.identifier(include_alias=True) for model_in_list in self.models)
+
+    def __getitem__(self, key: Union[int, ModelInfo]) -> ModelInfo:
+        """
+        find a model within list based on key or ModelInfo.
+        :param key: ModelInfo or index
+        :return: the model_info
+        """
+        if isinstance(key, int):
+            return self.models[key]
+
+        elif isinstance(key, ModelInfo):
+            identifier = key.identifier(include_alias=True)
+            for model in self.models:
+                if model.identifier(include_alias=True) == identifier:
+                    return model
+            raise KeyError(f"model with identifier '{identifier}' not found within this ModelList.")
+
+        else:
+            raise TypeError("ModelList indices must be int or ModelInfo.")
+
+    def append(self, model: ModelInfo):
+        self.models.append(model)
+
+    def _process_info(self):
+        """
+        apply all info within the model list (setting model active, ...) to keep data coherence
+        :return:
+        """
+        for identifier in self.identifiers_info:
+            if self.identifiers_info[identifier].active_alias is not None:
+                for model in self.models:
+                    if model.identifier(include_alias=False) == identifier and model.alias == self.identifiers_info[identifier].active_alias:
+                        model.is_active = True
+                    elif model.identifier(include_alias=False) == identifier:
+                        model.is_active = False
+
+
 class MLModelConfig(ApiDto):
     """
     a model config defines execution properties within a pipeline.
     usually to define how a pipeline should train and predict with your model.
 
+    :ivar format: by default 'pkl' for pickle, accept 'pt' for PyTorch model using tensors (none = 'pkl')
     :ivar bool by_twin: define if pipeline need to train and use different model by twin item.
     :ivar bool by_property: define if pipeline need to train and use different model based on 'property_name'.
     :ivar list features: datapoint list to refine columns if necessary.
@@ -23,6 +347,7 @@ class MLModelConfig(ApiDto):
     :ivar str model_key: key of the model to store (or use story property if dynamic).
     :ivar str model_type: reserved for 'mlflow' defines model flavor such as 'sklearn' or 'pyfunc'.
     :ivar str model_alias: reserved for 'mlflow' set the alias to target and use inside a pipeline.
+    :ivar str model_format: type of model (by default 'pkl') accept also 'pt' for PyTorch model using tensors (none = 'pkl')
     :ivar dict properties_mapping: dict to map properties expected by the script with the properties from the context.
     :ivar str property_name: define which property is looked for when using 'by_property'.
     :ivar str source: define name of model storage to use 'wizata'.
@@ -49,6 +374,7 @@ class MLModelConfig(ApiDto):
                  model_key: str = None,
                  model_type: str = None,
                  model_alias: str = None,
+                 model_format: str = 'pkl',
                  by_twin: bool = False,
                  by_property: bool = False,
                  property_name: str = None,
@@ -74,12 +400,38 @@ class MLModelConfig(ApiDto):
         self.by_twin = by_twin
         self.by_property = by_property
         self.property_name = property_name
+        self.model_format = model_format
 
         # source
         self.source = source
         self.model_type = model_type
         self.model_alias = model_alias
 
+    def create_model_info(self,
+                          hardware_id: str = None,
+                          property_value: str = None) -> ModelInfo:
+        """
+        create model info corresponding to the configuration.
+        :param hardware_id: provide a hardware id for this model if by_twin.
+        :param property_value: provide a value for this model if by_property.
+        :return:
+        """
+        if self.by_twin and hardware_id is None:
+            raise ValueError('hardware_id is required if by_twin to create a model info')
+        if self.by_property and property_value is None:
+            raise ValueError('property_value is required if by_property to create a model info')
+        model_info = ModelInfo(
+            key=self.model_key,
+            twin_hardware_id=hardware_id,
+            property_value=property_value,
+            source=self.source,
+            alias=self.model_alias,
+            file_format=self.model_format,
+            model_type=self.model_type,
+            property_name=self.property_name
+        )
+        return model_info
+
     def from_json(self, obj):
 
         # Managed deprecated fields
@@ -156,6 +508,8 @@ class MLModelConfig(ApiDto):
             self.model_type = obj["model_type"]
         if "model_alias" in obj.keys() and obj["model_alias"] is not None:
             self.model_alias = obj["model_alias"]
+        if "model_format" in obj.keys() and obj["model_format"] is not None:
+            self.model_format = obj["model_format"]
 
     def to_json(self, target: str = None):
         obj = {
@@ -199,6 +553,8 @@ class MLModelConfig(ApiDto):
             obj["by_property"] = str(self.by_property)
         if self.property_name is not None:
             obj["property_name"] = self.property_name
+        if self.model_format  is not None:
+            obj["model_format"] = self.model_format
 
         return obj
 
@@ -219,124 +575,3 @@ class MLModelConfig(ApiDto):
         else:
             raise TypeError(f'unsupported target_feat type {self.target_feat.__class__.__name__}')
 
-
-class MLModel(ApiDto):
-    """
-    used to define a stored machine learning model within Wizata.
-    - deprecated -
-    """
-
-    @classmethod
-    def route(cls):
-        return "mlmodels"
-
-    @classmethod
-    def from_dict(cls, data):
-        obj = MLModel()
-        obj.from_json(data)
-        return obj
-
-    @classmethod
-    def get_type(cls):
-        return "pickle"
-
-    def __init__(self,
-                 model_id: str = None,
-                 generated_by_id=None,
-                 exact_names=True,
-                 exact_numbers=True,
-                 key=None):
-        self.model_id = model_id
-        self.key = key
-
-        self.generatedById = generated_by_id
-
-        self.status = 'draft'
-        self.input_columns = []
-        self.output_columns = []
-
-        self.needExactColumnNumbers = exact_numbers
-        self.needExactColumnNames = exact_names
-        self.has_anomalies = False
-        self.label_counts = 0
-        self.has_target_feat = False
-
-        self.trained_model = None
-        self.scaler = None
-
-        self.experimentId = None
-
-    def api_id(self) -> str:
-        return str(self.model_id).upper()
-
-    def endpoint(self) -> str:
-        return "MLModels"
-
-    def to_json(self, target: str = None):
-        obj = {"id": str(self.model_id),
-               "status": str(self.status),
-               "needExactColumnNames": str(self.needExactColumnNames),
-               "needExactColumnNumbers": str(self.needExactColumnNumbers),
-               "hasAnomalies": str(self.has_anomalies),
-               "hasTargetFeat": str(self.has_target_feat),
-               "labelCount": str(self.label_counts)
-               }
-        if self.key is not None:
-            obj["key"] = str(self.key)
-        if self.generatedById is not None:
-            obj["generatedById"] = self.generatedById
-        if self.input_columns is not None:
-            obj["inputColumns"] = json.dumps(list(self.input_columns))
-        if self.output_columns is not None:
-            obj["outputColumns"] = json.dumps(list(self.output_columns))
-        if self.experimentId is not None:
-            obj["experimentId"] = str(self.experimentId)
-        return obj
-
-    def from_json(self, obj):
-        if "id" in obj.keys():
-            self.model_id = obj["id"]
-        if "key" in obj.keys() and obj["key"] is not None:
-            self.key = obj["key"]
-        if "generatedById" in obj.keys() and obj["generatedById"] is not None:
-            self.generatedById = int(obj["generatedById"])
-        if "experimentId" in obj.keys() and obj["experimentId"] is not None:
-            self.experimentId = uuid.UUID(obj["experimentId"])
-        if "status" in obj.keys():
-            self.status = str(obj["status"]).lower()
-        if "inputColumns" in obj.keys():
-            self.input_columns = json.loads(obj["inputColumns"])
-        if "outputColumns" in obj.keys():
-            self.output_columns = json.loads(obj["outputColumns"])
-        if "labelCount" in obj.keys():
-            self.label_counts = int(obj["labelCount"])
-        if "hasAnomalies" in obj.keys():
-            if isinstance(obj["hasAnomalies"], str) and obj["hasAnomalies"].lower() == "false":
-                self.has_anomalies = False
-            else:
-                self.has_anomalies = bool(obj["hasAnomalies"])
-        if "hasTargetFeat" in obj.keys():
-            if isinstance(obj["hasTargetFeat"], str) and obj["hasTargetFeat"].lower() == "false":
-                self.has_target_feat = False
-            else:
-                self.has_target_feat = bool(obj["hasTargetFeat"])
-        if "needExactColumnNumbers" in obj.keys():
-            if isinstance(obj["needExactColumnNumbers"], str) and obj["needExactColumnNumbers"].lower() == "false":
-                self.needExactColumnNumbers = False
-            else:
-                self.needExactColumnNumbers = bool(obj["needExactColumnNumbers"])
-        if "needExactColumnNames" in obj.keys():
-            if isinstance(obj["needExactColumnNames"], str) and obj["needExactColumnNames"].lower() == "false":
-                self.needExactColumnNames = False
-            else:
-                self.needExactColumnNames = bool(obj["needExactColumnNames"])
-
-    def get_sample_payload(self):
-        pl_columns = {"timestamp": "[timestamp]"}
-        for hardwareId in self.input_columns:
-            pl_columns[hardwareId] = "[" + hardwareId + "]"
-        pl_json = {
-            "id": str(self.model_id),
-            "dataset": pl_columns
-        }
-        return pl_json
