@@ -1,10 +1,8 @@
 import uuid
 from datetime import datetime, timedelta, timezone
-
-from reactivex.operators import group_by
-
 from .dataframe_toolkit import generate_epoch
 from .api_dto import ApiDto
+from .datapoint import BusinessType
 
 filter_map = {
     '>': 'gt',
@@ -26,12 +24,16 @@ class DynamicSelector:
     def __init__(self,
                  category = None,
                  unit = None,
+                 twin_type = None,
                  agg_method = None,
-                 rename = None):
+                 rename = None,
+                 business_type: BusinessType = None):
         self.category = category
         self.unit = unit
+        self.twin_type = twin_type
         self.agg_method = agg_method
         self.rename = rename
+        self.business_type = business_type
         self.datapoints = []
 
     @classmethod
@@ -47,6 +49,10 @@ class DynamicSelector:
             self.category = obj["category"]
         if "unit" in obj:
             self.unit = obj["unit"]
+        if "twin_type" in obj:
+            self.twin_type = obj["twin_type"]
+        if "business_type" in obj:
+            self.business_type = BusinessType(obj["business_type"])
         if "datapoints" in obj:
             self.datapoints = obj["datapoints"]
         if "rename" in obj:
@@ -60,6 +66,10 @@ class DynamicSelector:
             obj["category"] = self.category
         if self.unit:
             obj["unit"] = self.unit
+        if self.twin_type:
+            obj["twin_type"] = self.twin_type
+        if self.business_type:
+            obj["business_type"] = self.business_type.value
         if self.datapoints:
             obj["datapoints"] = self.datapoints
         if self.agg_method:
@@ -345,6 +355,36 @@ class Request(ApiDto):
         else:
             return
 
+    def set_twin(self, twin):
+        """
+        set a twin properly (str hardware Id or uuid.UUID)
+        :param twin:
+        :return:
+        """
+        if self.template is None:
+            self.template = {}
+        if isinstance(twin, uuid.UUID):
+            self.template['twin_id'] = twin
+        elif isinstance(twin, str):
+            self.template['twin_hardware_id'] = twin
+        else:
+            raise TypeError('twin must be str or uuid on request')
+
+    def set_template(self, template):
+        """
+        set a template properly (str key or uuid.UUID)
+        :param template:
+        :return:
+        """
+        if self.template is None:
+            self.template = {}
+        if isinstance(template, uuid.UUID):
+            self.template['template_id'] = template
+        elif isinstance(template, str):
+            self.template['template_key'] = template
+        else:
+            raise TypeError('template must be str or uuid on request')
+
     def __format_date(self, dt_to_format):
         if isinstance(dt_to_format, datetime):
             millisec = dt_to_format.timestamp() * 1000
@@ -458,22 +498,27 @@ class Request(ApiDto):
         else:
             self.template = {}
 
-            if template_id is not None:
+            template = None
+            if template_key is not None:
+                template = template_key
+            elif template_id is not None:
                 if isinstance(template_id, uuid.UUID):
-                    self.template['template_id'] = template_id
+                    template = template_id
                 else:
-                    self.template['template_id'] = uuid.UUID(template_id)
-            elif template_key is not None:
-                self.template['template_key'] = str(template_key)
+                    template = uuid.UUID(template_id)
+            if template is not None:
+                self.set_template(template=template)
 
-            if twin_id is not None:
+            twin = None
+            if twin_hardware_id is not None:
+                twin = twin_hardware_id
+            elif twin_id is not None:
                 if isinstance(twin_id, uuid.UUID):
-                    self.template['twin_id'] = twin_id
+                    twin = twin_id
                 else:
-                    self.template['twin_id'] = uuid.UUID(twin_id)
-
-            elif twin_hardware_id is not None:
-                self.template['twin_hardware_id'] = str(twin_hardware_id)
+                    twin = uuid.UUID(twin_id)
+            if twin is not None:
+                self.set_twin(twin=twin)
 
     def get_params(self):
         """
