@@ -23,7 +23,7 @@ from .api_dto import ApiDto, VarType
 from .business_label import BusinessLabel
 from .plot import Plot
 from .request import Request
-from .mlmodel import MLModel
+from .mlmodel import ModelInfo, ModelList
 from .experiment import Experiment
 from .script import Script
 from .execution import Execution, ExecutionStatus
@@ -590,15 +590,6 @@ class WizataDSAPIClient(ApiInterface):
                 return obj.script_id
             else:
                 raise self.__raise_error(response)
-        if isinstance(obj, MLModel):
-            response = requests.put(self.__url() + "mlmodels/",
-                                    headers=self.__header(),
-                                    data=pickle.dumps(obj))
-            if response.status_code == 200:
-                obj.model_id = uuid.UUID(response.json()['id'])
-                return obj.model_id
-            else:
-                raise self.__raise_error(response)
         if isinstance(obj, Template):
             return self.upsert_template(obj.key, obj.name)
         if isinstance(obj, Pipeline):
@@ -847,13 +838,6 @@ class WizataDSAPIClient(ApiInterface):
                 execution.script = self.get(script_name=script)
             elif isinstance(script, Script):
                 execution.script = script
-        if ml_model is not None:
-            if isinstance(ml_model, uuid.UUID) or (isinstance(ml_model, str) and is_valid_uuid(ml_model)):
-                execution.ml_model = MLModel(ml_model)
-            elif isinstance(ml_model, str):
-                execution.ml_model = self.get(model_key=ml_model)
-            elif isinstance(ml_model, MLModel):
-                execution.ml_model = ml_model
         if image is not None:
             execution.pipeline_image_id = image
         if isAnomalyDetection:
@@ -899,9 +883,6 @@ class WizataDSAPIClient(ApiInterface):
                 if "plots" in obj.keys():
                     for plot in obj["plots"]:
                         result_execution.plots.append(self.get(Plot(plot_id=plot["id"])))
-                if "models" in obj.keys():
-                    for mlmodel in obj["models"]:
-                        result_execution.models.append(self.get(MLModel(model_id=mlmodel["id"])))
                 if "resultDataframe" in obj.keys() and obj["resultDataframe"]["id"] is not None:
                     result_execution.output_ds_dataframe = self.get(DSDataFrame(df_id=obj["resultDataframe"]["id"]))
 
@@ -1097,56 +1078,48 @@ class WizataDSAPIClient(ApiInterface):
         else:
             raise self.__raise_error(response)
 
-    def register_model(self,
-                       model_key,
-                       train_model,
-                       df: pandas.DataFrame,
-                       scaler=None,
-                       has_anomalies: bool = False,
-                       has_target_feat: bool = False,
-                       experiment_key = None) -> tuple[MLModel, pandas.DataFrame]:
+    def upload_model(self,
+                     model_info: ModelInfo):
         """
-        Register a Machine Learning model to Wizata.
-        Model is tested by the API against a sample dataframe.
-        :param model_key: logical string id to identify the model.
-        :param train_model: trained model (must be compatible with pickle library)
-        :param df: sample dataframe.
-        :param scaler: scaler (must be compatible with pickle library)
-        :param has_anomalies: True is model generate Anomalies
-        :param has_target_feat: True if model need a target feature to be selected
-        :param experiment_key: Reference of an experiment to which link the generated ML Model
-        :return: registered ML Model , pandas.DataFrame
+        upload a model within the model repository.
+        :param model_info: model info, with at least key (+twin, +property, +alias) and trained_model.
         """
-        if model_key is None and isinstance(model_key, str):
-            raise ValueError('Please provide a str Model Key to identify the model.')
-        if train_model is None:
-            raise ValueError('Trained Machine Learning model should not be null')
-        elif df is None:
-            raise ValueError('A sample dataframe must be provided')
-
-        # Create a ML Model object
-        ml_model = MLModel()
-        ml_model.trained_model = train_model
-        if scaler is not None:
-            ml_model.scaler = scaler
-        ml_model.has_anomalies = has_anomalies
-        ml_model.has_target_feat = has_target_feat
-        ml_model.input_columns = df.columns
-        ml_model.key = model_key
-
-        if experiment_key is not None:
-            ml_model.experimentId = self.get(experiment_key=experiment_key).experiment_id
-
-        try:
-            result_df = predict(df, ml_model)
-            if result_df is not None:
-                ml_model.status = "valid"
-                self.upsert(ml_model)
-                return ml_model, result_df
-            else:
-                raise RuntimeError('no dataframe was generated by your model while testing predict capabilities')
-        except Exception as e:
-            raise RuntimeError('not able to validated the model : ' + str(e))
+        if model_info.trained_model is None:
+            raise ValueError("model_info must have a trained model as bytes content pkl or pt")
+        files = {
+            "trained_model": (
+                "trained_model." + model_info.file_format ,
+                model_info.trained_model,
+                "application/octet-stream",
+            )
+        }
+        headers = self.__header()
+        headers.pop("Content-Type", None)
+        response = requests.post(self.__url() + f"models",
+                                 headers=headers,
+                                 data={
+                                    "payload": json.dumps(model_info.to_json())
+                                 },
+                                 files=files)
+        if response.status_code == 200:
+            response_json = response.json()
+            if "identifier" in response_json:
+                key , twin, property_value, alias = ModelInfo.split_identifier(response_json["identifier"])
+                if alias is not None:
+                    model_info.alias = alias
+            for file in model_info.files:
+                file: wizata_dsapi.ModelFile
+                if file.name != "trained_model":
+                    headers["Content-Type"] = "application/octet-stream"
+                    response = requests.post(self.__url() + f"models/{model_info.identifier(include_alias=True)}/files/{file.path}",
+                                             headers=headers,
+                                             data=file.content,
+                                             timeout=60)
+                    if response.status_code != 200 and response.status_code != 201:
+                        raise self.__raise_error(response)
+            return model_info
+        else:
+            raise self.__raise_error(response)
 
     def upsert_experiment(self, key: str, name: str, pipeline=None, twin=None):
         """
